'''
This python script should be used in addition to the introduction of the Machine Learning Course.
Study of a base concept in ML: tradeoff between complexity of the model and the size of dataset to avoid under/over fitting.
Parametric Machine Learning consists in estimating the parameters of a fonction according to a training dataset. The resulting model should be able to infer new samples with the best possible accuracy. The input/output system to estimate (eg. classification, regression) has it own complexity. However, training a model needs : 1) an rough estimation of the complexity and 2) A large enough dataset according to this complexity. If one of both of these points is not correcty known, several failure cases may occur: 
1) if the complexity of the model is too small according to the true complexity of the problem, the model won't be able to estimate it. for exemple, a linear model (straight line) is unable to estimate a sinus process (even with 1M samples)
2) if the complexity of the model is too large according to the process, over-fitting can occur 
3) if the dataset is too small to explain the non linearity of the system, the resulting model cannot generalize on unseen inputs.

Questions: 

- Display, for several size of the training set, the accuracy of the training dataset (illustration of 1 and 3). 
- Display, for several size of the training set, the accuracy of the validation dataset: conclude (illustration of 1 and 3)
- Display, for several values of the complexity of the model, the accuracy of the training and testing set. (Illustration of 1)) 


'''
# Test1: evaluation of the size of training on classification (Knn)
from sklearn.datasets import load_digits
from sklearn import metrics
import numpy as np 
import matplotlib.pyplot as plt
from sklearn.model_selection import train_test_split
from sklearn.neighbors import KNeighborsClassifier
from sklearn.ensemble import AdaBoostClassifier
from sklearn.tree import DecisionTreeClassifier

import pdb
# General options
#
#
# Turn display_dataset to True to display image samples of the dataset
display_dataset=False
##
# Change this table to balance the size of the training dataset
##
train_ratio= [0.02,0.05,0.1,0.2,0.8] # part of the dataset used for training
##
# Feel free to change model_complexity
model_complexity = 100 # complexity of adaboost classifier (up to 1000 to avoid long training time)
##
#
# Load dataset
digits = load_digits()

if display_dataset:
  # Print to show there are 1797 images (8 by 8 images for a dimensionality of 64)
  print("Image data shape", digits.data.shape)
  # Print to show there are 1797 labels (integers from 0–9)
  print("Label Data Shape", digits.target.shape)
  plt.figure(figsize=(20,4))
  for index, (image, label) in enumerate(zip(digits.data[0:5], digits.target[0:5])):
    plt.subplot(1, 5, index + 1)
    plt.imshow(np.reshape(image, (8,8)), cmap=plt.cm.gray)
    plt.title('Training: %i\n' % label, fontsize = 20)
  plt.show()
# Divide dataset into train, valid, test 
x_train, x_val, y_train, y_val = train_test_split(digits.data, digits.target, test_size=0.25, random_state=0)
# all parameters not specified are set to their defaults
#neigh = KNeighborsClassifier(n_neighbors=3)
classifier = AdaBoostClassifier(
    DecisionTreeClassifier(max_depth=2),
    n_estimators=model_complexity,
    learning_rate=1.5,
    algorithm="SAMME")

Tscore = []
Tscore_train = []
Snbtrain = []
Lcm = []
for ratio in train_ratio:
  nbtrain = int(ratio*x_train.shape[0])
  x_trainr =x_train[0:nbtrain,:]
  y_trainr =y_train[0:nbtrain]
  classifier.fit(x_trainr, y_trainr)
  predictions = classifier.predict(x_val)
  predictions_train = classifier.predict(x_trainr)
  # Use score method to get accuracy of model
  score = classifier.score(x_val, y_val)
  score_train =classifier.score(x_trainr, y_trainr) 
  print(" val accuracy = {}. for {} training samples ({})".format(score,nbtrain,score_train))
  Tscore.append(score)
  Tscore_train.append(score_train)
  Snbtrain.append(nbtrain)
  cm = metrics.confusion_matrix(y_val, predictions)
  Lcm.append(cm)
  print(cm)
# Display results
plt.plot(Snbtrain,Tscore,'-g',label='val dataset')
plt.plot(Snbtrain,Tscore_train,'-r',label='train dataset')
plt.xlabel('Size of dataset')
plt.ylabel('Accuracy')
plt.legend()
plt.show()



